@@ -1,6 +1,7 @@
 #!/usr/bin/python3 -B
 import argparse
 import json
+import logging
 import os
 import shutil
 import stat
@@ -661,6 +662,7 @@ def _run_tests_for_plugin(context: CommandContext) -> None:
     "dh-integration-generate-debs",
     help_description="[Internal command] Generate .deb/.udebs packages from debian/<pkg> (Not stable API)",
     requested_plugins_only=True,
+    default_log_level=logging.WARN,
     argparser=[
         _add_packages_args,
         add_arg(
@@ -696,10 +698,14 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
             _error(
                 f"Plugins are not supported in the zz-debputy-rrr sequence. Detected plugins: {plugin_names}"
             )
+    debug_materialization = (
+        os.environ.get("DH_VERBOSE", "") != "" or parsed_args.debug_mode
+    )
 
     plugins = context.load_plugins().plugin_data
     for plugin in plugins.values():
-        _info(f"Loaded plugin {plugin.plugin_name}")
+        if not plugin.is_bundled:
+            _info(f"Loaded plugin {plugin.plugin_name}")
     manifest = context.parse_manifest()
 
     package_data_table = manifest.perform_installations(
@@ -763,6 +769,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
         manifest,
         package_data_table,
         is_dh_rrr_only_mode,
+        debug_materialization=debug_materialization,
     )
 
 
