@@ -1189,7 +1189,7 @@ def _render_value(v: Any) -> str:
     return str(v)
 
 
-def ensure_plugin_commands_are_loaded():
+def ensure_plugin_commands_are_loaded() -> None:
     # Loading the module does the heavy lifting
     # However, having this function means that we do not have an "unused" import that some tool
     # gets tempted to remove
