@@ -8,6 +8,7 @@ import itertools
 import operator
 import os
 import re
+import shutil
 import subprocess
 import tempfile
 import textwrap
@@ -698,12 +699,17 @@ def _attach_debug(objcopy: str, elf_binary: VirtualPath, dbgsym: FSPath) -> None
             )
 
 
+@functools.lru_cache()
+def _has_tool(tool: str) -> bool:
+    return shutil.which(tool) is not None
+
+
 def _run_dwz(
     dctrl: BinaryPackage,
     dbgsym_fs_root: FSPath,
     unstripped_elf_info: List[_ElfInfo],
 ) -> None:
-    if not unstripped_elf_info or dctrl.is_udeb:
+    if not unstripped_elf_info or dctrl.is_udeb or not _has_tool("dwz"):
         return
     dwz_cmd = ["dwz"]
     dwz_ma_dir_name = f"usr/lib/debug/.dwz/{dctrl.deb_multiarch}"
@@ -930,7 +936,7 @@ def _relevant_service_definitions(
         if key in by_service_manager_key
         and service_rule.applies_to_service_manager(key[-1])
     }
-    relevant_names = {}
+    relevant_names: Dict[Tuple[str, str, str, str], ServiceDefinition[Any]] = {}
     seen_keys = set()
 
     if not pending_queue:
@@ -954,7 +960,7 @@ def _relevant_service_definitions(
                 ):
                     pending_queue.add(target_key)
 
-    return relevant_names
+    return relevant_names.items()
 
 
 def handle_service_management(
@@ -982,7 +988,9 @@ def handle_service_management(
                 )
 
     for service_manager_details in feature_set.service_managers.values():
-        service_registry = ServiceRegistryImpl(service_manager_details)
+        service_registry: ServiceRegistryImpl = ServiceRegistryImpl(
+            service_manager_details
+        )
         service_manager_details.service_detector(
             fs_root,
             service_registry,
@@ -1214,7 +1222,7 @@ def setup_control_files(
         return
 
     if generated_triggers:
-        assert not allow_ctrl_file_management
+        assert allow_ctrl_file_management
         dest_file = os.path.join(control_output_dir, "triggers")
         with open(dest_file, "at", encoding="utf-8") as fd:
             fd.writelines(
@@ -1652,6 +1660,7 @@ def _generate_control_files(
             dctrl_file = "debian/control"
 
         if has_dbgsym:
+            assert dbgsym_root_fs is not None  # mypy hint
             _generate_dbgsym_control_file_if_relevant(
                 binary_package,
                 dbgsym_root_fs,
