@@ -231,6 +231,11 @@ DH_ADDONS_TO_PLUGINS = {
         remove_dh_sequence=False,
         must_use_zz_debputy=True,
     ),
+    "grantlee": DHSequenceMigration(
+        "grantlee",
+        remove_dh_sequence=True,
+        must_use_zz_debputy=True,
+    ),
     "numpy3": DHSequenceMigration(
         "numpy3",
         # The sequence provides (build-time) dependencies that we cannot provide
@@ -432,7 +437,7 @@ def migrate_bash_completion(
                     install_as_rules.append((source, dest_basename))
 
             if install_dest_sources:
-                sources = (
+                sources: Union[List[str], str] = (
                     install_dest_sources
                     if len(install_dest_sources) > 1
                     else install_dest_sources[0]
@@ -1502,7 +1507,7 @@ def read_dh_addon_sequences(
     ctrl_file = debian_dir.get("control")
     if ctrl_file:
         dr_sequences: Set[str] = set()
-        bd_sequences = set()
+        bd_sequences: Set[str] = set()
 
         drules = debian_dir.get("rules")
         if drules and drules.is_file:
