@@ -437,14 +437,17 @@ class DebputyLanguageServer(LanguageServer):
     def determine_language_id(
         self,
         doc: "TextDocument",
-    ) -> Tuple[Literal["editor-provided", "filename"], str]:
+    ) -> Tuple[Literal["editor-provided", "filename"], str, str]:
         lang_id = doc.language_id
-        if self.trust_language_ids and lang_id and not lang_id.isspace():
-            return "editor-provided", lang_id
         path = doc.path
         try:
             last_idx = path.rindex("debian/")
         except ValueError:
-            return "filename", os.path.basename(path)
-        guess_language_id = path[last_idx:]
-        return "filename", guess_language_id
+            cleaned_filename = os.path.basename(path)
+        else:
+            cleaned_filename = path[last_idx:]
+
+        if self.trust_language_ids and lang_id and not lang_id.isspace():
+            return "editor-provided", lang_id, cleaned_filename
+
+        return "filename", cleaned_filename, cleaned_filename
