@@ -22,7 +22,11 @@ from lsprotocol.types import (
 
 from debputy.linting.lint_util import LintState
 from debputy.lsp.diagnostics import DiagnosticData
-from debputy.lsp.lsp_features import lsp_diagnostics, lsp_standard_handler
+from debputy.lsp.lsp_features import (
+    lsp_diagnostics,
+    lsp_standard_handler,
+    LanguageDispatch,
+)
 from debputy.lsp.quickfixes import (
     propose_correct_text_quick_fix,
 )
@@ -45,11 +49,11 @@ except ImportError:
 _MAXIMUM_WIDTH: int = 82
 _HEADER_LINE = re.compile(r"^(\S+)\s*[(]([^)]+)[)]")  # TODO: Add reset
 _LANGUAGE_IDS = [
-    "debian/changelog",
+    LanguageDispatch.from_language_id("debian/changelog"),
     # emacs's name
-    "debian-changelog",
+    LanguageDispatch.from_language_id("debian-changelog"),
     # vim's name
-    "debchangelog",
+    LanguageDispatch.from_language_id("debchangelog"),
 ]
 
 _WEEKDAYS_BY_IDX = [
