@@ -9,7 +9,6 @@ from typing import (
     Mapping,
     List,
     Dict,
-    Any,
 )
 
 from lsprotocol.types import (
@@ -55,6 +54,7 @@ from debputy.lsp.lsp_features import (
     lsp_semantic_tokens_full,
     lsp_will_save_wait_until,
     lsp_format_document,
+    LanguageDispatch,
 )
 from debputy.lsp.lsp_generic_deb822 import (
     deb822_completer,
@@ -85,8 +85,6 @@ from debputy.lsp.vendoring._deb822_repro import (
 from debputy.lsp.vendoring._deb822_repro.parsing import (
     Deb822KeyValuePairElement,
     LIST_SPACE_SEPARATED_INTERPRETATION,
-    Interpretation,
-    Deb822ParsedTokenList,
 )
 
 try:
@@ -103,11 +101,11 @@ except ImportError:
 
 
 _LANGUAGE_IDS = [
-    "debian/control",
+    LanguageDispatch.from_language_id("debian/control"),
     # emacs's name
-    "debian-control",
+    LanguageDispatch.from_language_id("debian-control"),
     # vim's name
-    "debcontrol",
+    LanguageDispatch.from_language_id("debcontrol"),
 ]
 
 
