@@ -19,8 +19,6 @@ from lsprotocol.types import (
     DiagnosticSeverity,
     HoverParams,
     Hover,
-    MarkupKind,
-    MarkupContent,
     TEXT_DOCUMENT_CODE_ACTION,
     CompletionParams,
     CompletionList,
@@ -29,43 +27,34 @@ from lsprotocol.types import (
     Location,
 )
 
+from debputy.highlevel_manifest import MANIFEST_YAML
 from debputy.linting.lint_util import LintState
 from debputy.lsp.diagnostics import DiagnosticData
-from debputy.lsp.quickfixes import propose_correct_text_quick_fix
-from debputy.manifest_parser.base_types import DebputyDispatchableType
-from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
-from debputy.yaml.compat import (
-    Node,
-    CommentedMap,
-    LineCol,
-    CommentedSeq,
-    CommentedBase,
-    MarkedYAMLError,
-    YAMLError,
-)
-
-from debputy.highlevel_manifest import MANIFEST_YAML
 from debputy.lsp.lsp_features import (
     lint_diagnostics,
     lsp_standard_handler,
     lsp_hover,
     lsp_completer,
+    LanguageDispatch,
 )
+from debputy.lsp.lsp_generic_yaml import (
+    resolve_hover_text,
+    as_hover_doc,
+    is_before,
+    word_range_at_position,
+)
+from debputy.lsp.quickfixes import propose_correct_text_quick_fix
 from debputy.lsp.text_util import (
     LintCapablePositionCodec,
     detect_possible_typo,
 )
+from debputy.manifest_parser.base_types import DebputyDispatchableType
 from debputy.manifest_parser.declarative_parser import (
     AttributeDescription,
     ParserGenerator,
     DeclarativeNonMappingInputParser,
 )
 from debputy.manifest_parser.declarative_parser import DeclarativeMappingInputParser
-from debputy.manifest_parser.parser_doc import (
-    render_rule,
-    render_attribute_doc,
-    doc_args_for_parser_doc,
-)
 from debputy.manifest_parser.util import AttributePath
 from debputy.plugin.api.impl import plugin_metadata_for_debputys_own_plugin
 from debputy.plugin.api.impl_types import (
@@ -77,8 +66,16 @@ from debputy.plugin.api.impl_types import (
     InPackageContextParser,
     DeclarativeValuelessKeywordInputParser,
 )
-from debputy.util import _info, _warn
-
+from debputy.util import _info
+from debputy.yaml.compat import (
+    Node,
+    CommentedMap,
+    LineCol,
+    CommentedSeq,
+    CommentedBase,
+    MarkedYAMLError,
+    YAMLError,
+)
 
 try:
     from pygls.server import LanguageServer
@@ -88,10 +85,12 @@ except ImportError:
 
 
 _LANGUAGE_IDS = [
-    "debian/debputy.manifest",
-    "debputy.manifest",
+    LanguageDispatch.from_language_id("debian/debputy.manifest"),
+    LanguageDispatch.from_language_id("debputy.manifest"),
     # LSP's official language ID for YAML files
-    "yaml",
+    LanguageDispatch.from_language_id(
+        "yaml", filename_selector="debian/debputy.manifest"
+    ),
 ]
 
 
@@ -99,42 +98,12 @@ lsp_standard_handler(_LANGUAGE_IDS, TEXT_DOCUMENT_CODE_ACTION)
 lsp_standard_handler(_LANGUAGE_IDS, TEXT_DOCUMENT_WILL_SAVE_WAIT_UNTIL)
 
 
-def is_valid_file(path: str) -> bool:
-    # For debian/debputy.manifest, the language ID is often set to makefile meaning we get random
-    # "non-debian/debputy.manifest" YAML files here. Skip those.
-    return path.endswith("debian/debputy.manifest")
-
-
-def _word_range_at_position(
-    lines: List[str],
-    line_no: int,
-    char_offset: int,
-) -> Range:
-    line = lines[line_no]
-    line_len = len(line)
-    start_idx = char_offset
-    end_idx = char_offset
-    while end_idx + 1 < line_len and not line[end_idx + 1].isspace():
-        end_idx += 1
-
-    while start_idx - 1 >= 0 and not line[start_idx - 1].isspace():
-        start_idx -= 1
-
-    return Range(
-        Position(line_no, start_idx),
-        Position(line_no, end_idx),
-    )
-
-
 @lint_diagnostics(_LANGUAGE_IDS)
 def _lint_debian_debputy_manifest(
     lint_state: LintState,
 ) -> Optional[List[Diagnostic]]:
     lines = lint_state.lines
     position_codec = lint_state.position_codec
-    path = lint_state.path
-    if not is_valid_file(path):
-        return None
     diagnostics: List[Diagnostic] = []
     try:
         content = MANIFEST_YAML.load("".join(lines))
@@ -147,7 +116,7 @@ def _lint_debian_debputy_manifest(
             column = e.problem_mark.column + 1
         error_range = position_codec.range_to_client_units(
             lines,
-            _word_range_at_position(
+            word_range_at_position(
                 lines,
                 line,
                 column,
@@ -191,7 +160,7 @@ def _lint_debian_debputy_manifest(
 
 
 def _unknown_key(
-    key: str,
+    key: Optional[str],
     expected_keys: Iterable[str],
     line: int,
     col: int,
@@ -200,6 +169,7 @@ def _unknown_key(
     *,
     message_format: str = 'Unknown or unsupported key "{key}".',
 ) -> Tuple["Diagnostic", Optional[str]]:
+    key_len = len(key) if key else 1
     key_range = position_codec.range_to_client_units(
         lines,
         Range(
@@ -209,12 +179,12 @@ def _unknown_key(
             ),
             Position(
                 line,
-                col + len(key),
+                col + key_len,
             ),
         ),
     )
 
-    candidates = detect_possible_typo(key, expected_keys)
+    candidates = detect_possible_typo(key, expected_keys) if key is not None else ()
     extra = ""
     corrected_key = None
     if candidates:
@@ -223,6 +193,8 @@ def _unknown_key(
         #  That would enable this to work in more cases.
         corrected_key = candidates[0] if len(candidates) == 1 else None
 
+    if key is None:
+        message_format = "Missing key"
     diagnostic = Diagnostic(
         key_range,
         message_format.format(key=key) + extra,
@@ -312,6 +284,9 @@ def _lint_attr_value(
     value: Any,
 ) -> Iterable["Diagnostic"]:
     attr_type = attr.attribute_type
+    type_mapping = pg.get_mapped_type_from_target_type(attr_type)
+    if type_mapping is not None:
+        attr_type = type_mapping.source_type
     orig = get_origin(attr_type)
     valid_values: Sequence[Any] = tuple()
     if orig == Literal:
@@ -449,7 +424,6 @@ def _lint_content(
     elif isinstance(parser, InPackageContextParser):
         if not isinstance(content, CommentedMap):
             return
-        print(lint_state)
         known_packages = lint_state.binary_packages
         lc = content.lc
         for k, v in content.items():
@@ -475,28 +449,6 @@ def _lint_content(
         )
 
 
-def is_at(position: Position, lc_pos: Tuple[int, int]) -> bool:
-    return position.line == lc_pos[0] and position.character == lc_pos[1]
-
-
-def is_before(position: Position, lc_pos: Tuple[int, int]) -> bool:
-    line, column = lc_pos
-    if position.line < line:
-        return True
-    if position.line == line and position.character < column:
-        return True
-    return False
-
-
-def is_after(position: Position, lc_pos: Tuple[int, int]) -> bool:
-    line, column = lc_pos
-    if position.line > line:
-        return True
-    if position.line == line and position.character > column:
-        return True
-    return False
-
-
 def _trace_cursor(
     content: Any,
     attribute_path: AttributePath,
@@ -629,59 +581,9 @@ def resolve_keyword(
     return None
 
 
-def _render_param_doc(
-    rule_name: str,
-    declarative_parser: DeclarativeMappingInputParser,
-    plugin_metadata: DebputyPluginMetadata,
-    attribute: str,
-) -> Optional[str]:
-    attr = declarative_parser.source_attributes.get(attribute)
-    if attr is None:
-        return None
-
-    doc_args, parser_doc = doc_args_for_parser_doc(
-        rule_name,
-        declarative_parser,
-        plugin_metadata,
-    )
-    rendered_docs = render_attribute_doc(
-        declarative_parser,
-        declarative_parser.source_attributes,
-        declarative_parser.input_time_required_parameters,
-        declarative_parser.at_least_one_of,
-        parser_doc,
-        doc_args,
-        is_interactive=True,
-        rule_name=rule_name,
-    )
-
-    for attributes, rendered_doc in rendered_docs:
-        if attribute in attributes:
-            full_doc = [
-                f"# Attribute `{attribute}`",
-                "",
-            ]
-            full_doc.extend(rendered_doc)
-
-            return "\n".join(full_doc)
-    return None
-
-
 DEBPUTY_PLUGIN_METADATA = plugin_metadata_for_debputys_own_plugin()
 
 
-def _guess_rule_name(segments: List[Union[str, int]], idx: int) -> str:
-    orig_idx = idx
-    idx -= 1
-    while idx >= 0:
-        segment = segments[idx]
-        if isinstance(segment, str):
-            return segment
-        idx -= 1
-    _warn(f"Unable to derive rule name from {segments} [{orig_idx}]")
-    return "<Bug: unknown rule name>"
-
-
 def _escape(v: str) -> str:
     return '"' + v.replace("\n", "\\n") + '"'
 
@@ -698,23 +600,27 @@ def _insert_snippet(lines: List[str], server_position: Position) -> bool:
     lhs = lhs_ws.strip()
     if lhs.endswith(":"):
         _info("Insertion of value (key seen)")
-        new_line = line[: server_position.character] + _COMPLETION_HINT_VALUE
+        new_line = line[: server_position.character] + _COMPLETION_HINT_VALUE + "\n"
     elif lhs.startswith("-"):
         _info("Insertion of key or value (list item)")
         # Respect the provided indentation
         snippet = _COMPLETION_HINT_KEY if ":" not in lhs else _COMPLETION_HINT_VALUE
-        new_line = line[: server_position.character] + snippet
+        new_line = line[: server_position.character] + snippet + "\n"
     elif not lhs or (lhs_ws and not lhs_ws[0].isspace()):
         _info(f"Insertion of key or value: {_escape(line[server_position.character:])}")
         # Respect the provided indentation
         snippet = _COMPLETION_HINT_KEY if ":" not in lhs else _COMPLETION_HINT_VALUE
-        new_line = line[: server_position.character] + snippet
+        new_line = line[: server_position.character] + snippet + "\n"
     elif lhs.isalpha() and ":" not in lhs:
         _info(f"Expanding value to a key: {_escape(line[server_position.character:])}")
         # Respect the provided indentation
-        new_line = line[: server_position.character] + _COMPLETION_HINT_KEY
+        new_line = line[: server_position.character] + _COMPLETION_HINT_KEY + "\n"
     else:
-        c = line[server_position.character]
+        c = (
+            line[server_position.character]
+            if server_position.character < len(line)
+            else "(OOB)"
+        )
         _info(f"Not touching line: {_escape(line)} -- {_escape(c)}")
         return False
     _info(f'Evaluating complete on synthetic line: "{new_line}"')
@@ -728,13 +634,13 @@ def debputy_manifest_completer(
     params: CompletionParams,
 ) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
     doc = ls.workspace.get_text_document(params.text_document.uri)
-    if not is_valid_file(doc.path):
-        return None
     lines = doc.lines
     server_position = doc.position_codec.position_from_client_units(
         lines, params.position
     )
     attribute_root_path = AttributePath.root_path()
+    orig_line = lines[server_position.line].rstrip()
+    has_colon = ":" in orig_line
     added_key = _insert_snippet(lines, server_position)
     attempts = 1 if added_key else 2
     content = None
@@ -800,7 +706,7 @@ def debputy_manifest_completer(
         if isinstance(parser, DispatchingParserBase):
             if matched_key:
                 items = [
-                    CompletionItem(f"{k}:")
+                    CompletionItem(k if has_colon else f"{k}:")
                     for k in parser.registered_keywords()
                     if k not in parent
                     and not isinstance(
@@ -822,7 +728,9 @@ def debputy_manifest_completer(
             binary_packages = ls.lint_state(doc).binary_packages
             if binary_packages is not None:
                 items = [
-                    CompletionItem(f"{p}:") for p in binary_packages if p not in parent
+                    CompletionItem(p if has_colon else f"{p}:")
+                    for p in binary_packages
+                    if p not in parent
                 ]
         elif isinstance(parser, DeclarativeMappingInputParser):
             if matched_key:
@@ -836,7 +744,7 @@ def debputy_manifest_completer(
                         locked.add(attr_name)
                         break
                 items = [
-                    CompletionItem(f"{k}:")
+                    CompletionItem(k if has_colon else f"{k}:")
                     for k in parser.manifest_attributes
                     if k not in locked
                 ]
@@ -870,10 +778,17 @@ def _completion_from_attr(
     pg: ParserGenerator,
     matched: Any,
 ) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
-    orig = get_origin(attr.attribute_type)
+    type_mapping = pg.get_mapped_type_from_target_type(attr.attribute_type)
+    if type_mapping is not None:
+        attr_type = type_mapping.source_type
+    else:
+        attr_type = attr.attribute_type
+
+    orig = get_origin(attr_type)
     valid_values: Sequence[Any] = tuple()
+
     if orig == Literal:
-        valid_values = get_args(attr.attribute_type)
+        valid_values = get_args(attr_type)
     elif orig == bool or attr.attribute_type == bool:
         valid_values = ("true", "false")
     elif isinstance(orig, type) and issubclass(orig, DebputyDispatchableType):
@@ -894,8 +809,6 @@ def debputy_manifest_hover(
     params: HoverParams,
 ) -> Optional[Hover]:
     doc = ls.workspace.get_text_document(params.text_document.uri)
-    if not is_valid_file(doc.path):
-        return None
     lines = doc.lines
     position_codec = doc.position_codec
     attribute_root_path = AttributePath.root_path()
@@ -937,100 +850,4 @@ def debputy_manifest_hover(
         matched,
         matched_key,
     )
-    return _hover_doc(ls, hover_doc_text)
-
-
-def resolve_hover_text_for_value(
-    feature_set: PluginProvidedFeatureSet,
-    parser: DeclarativeMappingInputParser,
-    plugin_metadata: DebputyPluginMetadata,
-    segment: Union[str, int],
-    matched: Any,
-) -> Optional[str]:
-
-    hover_doc_text: Optional[str] = None
-    attr = parser.manifest_attributes.get(segment)
-    attr_type = attr.attribute_type if attr is not None else None
-    if attr_type is None:
-        _info(f"Matched value for {segment} -- No attr or type")
-        return None
-    if isinstance(attr_type, type) and issubclass(attr_type, DebputyDispatchableType):
-        parser_generator = feature_set.manifest_parser_generator
-        parser = parser_generator.dispatch_parser_table_for(attr_type)
-        if parser is None or not isinstance(matched, str):
-            _info(
-                f"Unknown parser for {segment} or matched is not a str -- {attr_type} {type(matched)=}"
-            )
-            return None
-        subparser = parser.parser_for(matched)
-        if subparser is None:
-            _info(f"Unknown parser for {matched} (subparser)")
-            return None
-        hover_doc_text = render_rule(
-            matched,
-            subparser.parser,
-            plugin_metadata,
-        )
-    else:
-        _info(f"Unknown value: {matched} -- {segment}")
-    return hover_doc_text
-
-
-def resolve_hover_text(
-    feature_set: PluginProvidedFeatureSet,
-    parser: Optional[Union[DeclarativeInputParser[Any], DispatchingParserBase]],
-    plugin_metadata: DebputyPluginMetadata,
-    segments: List[Union[str, int]],
-    at_depth_idx: int,
-    matched: Any,
-    matched_key: bool,
-) -> Optional[str]:
-    hover_doc_text: Optional[str] = None
-    if at_depth_idx == len(segments):
-        segment = segments[at_depth_idx - 1]
-        _info(f"Matched {segment} at ==, {matched_key=} ")
-        hover_doc_text = render_rule(
-            segment,
-            parser,
-            plugin_metadata,
-            is_root_rule=False,
-        )
-    elif at_depth_idx + 1 == len(segments) and isinstance(
-        parser, DeclarativeMappingInputParser
-    ):
-        segment = segments[at_depth_idx]
-        _info(f"Matched {segment} at -1, {matched_key=} ")
-        if isinstance(segment, str):
-            if not matched_key:
-                hover_doc_text = resolve_hover_text_for_value(
-                    feature_set,
-                    parser,
-                    plugin_metadata,
-                    segment,
-                    matched,
-                )
-            if matched_key or hover_doc_text is None:
-                rule_name = _guess_rule_name(segments, at_depth_idx)
-                hover_doc_text = _render_param_doc(
-                    rule_name,
-                    parser,
-                    plugin_metadata,
-                    segment,
-                )
-    else:
-        _info(f"No doc: {at_depth_idx=} {len(segments)=}")
-
-    return hover_doc_text
-
-
-def _hover_doc(
-    ls: "DebputyLanguageServer", hover_doc_text: Optional[str]
-) -> Optional[Hover]:
-    if hover_doc_text is None:
-        return None
-    return Hover(
-        contents=MarkupContent(
-            kind=ls.hover_markup_format(MarkupKind.Markdown, MarkupKind.PlainText),
-            value=hover_doc_text,
-        ),
-    )
+    return as_hover_doc(ls, hover_doc_text)
