@@ -34,6 +34,7 @@ from debputy.lsp.lsp_features import (
     lint_diagnostics,
     lsp_standard_handler,
     lsp_completer,
+    LanguageDispatch,
 )
 from debputy.lsp.quickfixes import propose_correct_text_quick_fix
 from debputy.lsp.spellchecking import spellcheck_line
@@ -111,13 +112,15 @@ _COMMAND_WORDS = frozenset(
 )
 
 _LANGUAGE_IDS = [
-    "debian/rules",
+    LanguageDispatch.from_language_id("debian/rules"),
     # LSP's official language ID for Makefile
-    "makefile",
+    LanguageDispatch.from_language_id("makefile", filename_selector="debian/rules"),
     # emacs's name (there is no debian-rules mode)
-    "makefile-gmake",
+    LanguageDispatch.from_language_id(
+        "makefile-gmake", filename_selector="debian/rules"
+    ),
     # vim's name (there is no debrules)
-    "make",
+    LanguageDispatch.from_language_id("make", filename_selector="debian/rules"),
 ]
 
 
@@ -133,16 +136,8 @@ lsp_standard_handler(_LANGUAGE_IDS, TEXT_DOCUMENT_CODE_ACTION)
 lsp_standard_handler(_LANGUAGE_IDS, TEXT_DOCUMENT_WILL_SAVE_WAIT_UNTIL)
 
 
-def is_valid_file(path: str) -> bool:
-    # For debian/rules, the language ID is often set to makefile meaning we get random "non-debian/rules"
-    # makefiles here. Skip those.
-    return path.endswith("debian/rules")
-
-
 @lint_diagnostics(_LANGUAGE_IDS)
 def _lint_debian_rules(lint_state: LintState) -> Optional[List[Diagnostic]]:
-    if not is_valid_file(lint_state.path):
-        return None
     return _lint_debian_rules_impl(lint_state)
 
 
@@ -356,8 +351,6 @@ def _debian_rules_completions(
     params: CompletionParams,
 ) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
     doc = ls.workspace.get_text_document(params.text_document.uri)
-    if not is_valid_file(doc.path):
-        return None
     lines = doc.lines
     server_position = doc.position_codec.position_from_client_units(
         lines, params.position
