@@ -48,6 +48,7 @@ from debputy.lsp.lsp_features import (
     lsp_semantic_tokens_full,
     lsp_will_save_wait_until,
     lsp_format_document,
+    LanguageDispatch,
 )
 from debputy.lsp.lsp_generic_deb822 import (
     deb822_completer,
@@ -91,11 +92,11 @@ except ImportError:
 
 _CONTAINS_SPACE_OR_COLON = re.compile(r"[\s:]")
 _LANGUAGE_IDS = [
-    "debian/tests/control",
+    LanguageDispatch.from_language_id("debian/tests/control"),
     # emacs's name - expected in elpa-dpkg-dev-el (>> 37.11)
-    "debian-autopkgtest-control-mode",
+    LanguageDispatch.from_language_id("debian-autopkgtest-control-mode"),
     # Likely to be vim's name if it had support
-    "debtestscontrol",
+    LanguageDispatch.from_language_id("debtestscontrol"),
 ]
 
 _DTESTS_CTRL_FILE_METADATA = DTestsCtrlFileMetadata()
