@@ -1,4 +1,5 @@
 import collections
+import dataclasses
 import inspect
 import sys
 from typing import (
@@ -10,8 +11,13 @@ from typing import (
     List,
     Optional,
     AsyncIterator,
+    Self,
+    Generic,
 )
 
+from debputy.commands.debputy_cmd.context import CommandContext
+from debputy.commands.debputy_cmd.output import _output_styling
+from debputy.lsp.lsp_self_check import LSP_CHECKS
 from lsprotocol.types import (
     TEXT_DOCUMENT_WILL_SAVE_WAIT_UNTIL,
     TEXT_DOCUMENT_CODE_ACTION,
@@ -23,10 +29,6 @@ from lsprotocol.types import (
     SemanticTokenTypes,
 )
 
-from debputy.commands.debputy_cmd.context import CommandContext
-from debputy.commands.debputy_cmd.output import _output_styling
-from debputy.lsp.lsp_self_check import LSP_CHECKS
-
 try:
     from pygls.server import LanguageServer
     from debputy.lsp.debputy_ls import DebputyLanguageServer
@@ -52,15 +54,17 @@ SEMANTIC_TOKEN_TYPES_IDS = {
     t: idx for idx, t in enumerate(SEMANTIC_TOKENS_LEGEND.token_types)
 }
 
+DiagnosticHandler = Callable[
+    [
+        "DebputyLanguageServer",
+        Union["DidOpenTextDocumentParams", "DidChangeTextDocumentParams"],
+    ],
+    AsyncIterator[Optional[List[Diagnostic]]],
+]
+
 DIAGNOSTIC_HANDLERS: Dict[
     str,
-    Callable[
-        [
-            "DebputyLanguageServer",
-            Union["DidOpenTextDocumentParams", "DidChangeTextDocumentParams"],
-        ],
-        AsyncIterator[Optional[List[Diagnostic]]],
-    ],
+    List["_DispatchRule[DiagnosticHandler]"],
 ] = {}
 COMPLETER_HANDLERS = {}
 HOVER_HANDLERS = {}
@@ -71,6 +75,35 @@ WILL_SAVE_WAIT_UNTIL_HANDLERS = {}
 FORMAT_FILE_HANDLERS = {}
 _ALIAS_OF = {}
 
+
+@dataclasses.dataclass(slots=True, frozen=True)
+class LanguageDispatch:
+    language_id: Optional[str]
+    filename_selector: Optional[Union[str, Callable[[str], bool]]] = None
+
+    @classmethod
+    def from_language_id(
+        cls,
+        language_id: str,
+        filename_selector: Optional[Union[str, Callable[[str], bool]]] = None,
+    ) -> Self:
+        return cls(language_id, filename_selector=filename_selector)
+
+    def filename_match(self, filename: str) -> bool:
+        selector = self.filename_selector
+        if selector is None:
+            return True
+        if isinstance(selector, str):
+            return filename == selector
+        return selector(filename)
+
+
+@dataclasses.dataclass(slots=True, frozen=True)
+class _DispatchRule(Generic[C]):
+    language_dispatch: LanguageDispatch
+    handler: C
+
+
 _STANDARD_HANDLERS = {
     TEXT_DOCUMENT_FORMATTING: (
         FORMAT_FILE_HANDLERS,
@@ -88,7 +121,7 @@ _STANDARD_HANDLERS = {
 
 
 def lint_diagnostics(
-    file_formats: Union[str, Sequence[str]]
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
 ) -> Callable[[LinterImpl], LinterImpl]:
 
     def _wrapper(func: C) -> C:
@@ -109,18 +142,25 @@ def lint_diagnostics(
             raise ValueError("Linters are all non-async at the moment")
 
         for file_format in file_formats:
-            if file_format in DIAGNOSTIC_HANDLERS:
+            if file_format.language_id in DIAGNOSTIC_HANDLERS:
                 raise AssertionError(
                     "There is already a diagnostics handler for " + file_format
                 )
-            DIAGNOSTIC_HANDLERS[file_format] = _lint_wrapper
+            handler_metadata = _DispatchRule(file_format, _lint_wrapper)
+            handlers = DIAGNOSTIC_HANDLERS.get(file_format.language_id)
+            if handlers is None:
+                DIAGNOSTIC_HANDLERS[file_format.language_id] = [handler_metadata]
+            else:
+                handlers.append(handler_metadata)
 
         return func
 
     return _wrapper
 
 
-def lsp_diagnostics(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]:
+def lsp_diagnostics(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
+) -> Callable[[C], C]:
 
     def _wrapper(func: C) -> C:
 
@@ -145,35 +185,45 @@ def lsp_diagnostics(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]
     return _wrapper
 
 
-def lsp_completer(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]:
+def lsp_completer(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
+) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, COMPLETER_HANDLERS)
 
 
-def lsp_hover(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]:
+def lsp_hover(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
+) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, HOVER_HANDLERS)
 
 
-def lsp_folding_ranges(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]:
+def lsp_folding_ranges(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
+) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, FOLDING_RANGE_HANDLERS)
 
 
 def lsp_will_save_wait_until(
-    file_formats: Union[str, Sequence[str]]
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
 ) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, WILL_SAVE_WAIT_UNTIL_HANDLERS)
 
 
-def lsp_format_document(file_formats: Union[str, Sequence[str]]) -> Callable[[C], C]:
+def lsp_format_document(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
+) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, FORMAT_FILE_HANDLERS)
 
 
 def lsp_semantic_tokens_full(
-    file_formats: Union[str, Sequence[str]]
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]]
 ) -> Callable[[C], C]:
     return _registering_wrapper(file_formats, SEMANTIC_TOKENS_FULL_HANDLERS)
 
 
-def lsp_standard_handler(file_formats: Union[str, Sequence[str]], topic: str) -> None:
+def lsp_standard_handler(
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]], topic: str
+) -> None:
     res = _STANDARD_HANDLERS.get(topic)
     if res is None:
         raise ValueError(f"No standard handler for {topic}")
@@ -184,7 +234,8 @@ def lsp_standard_handler(file_formats: Union[str, Sequence[str]], topic: str) ->
 
 
 def _registering_wrapper(
-    file_formats: Union[str, Sequence[str]], handler_dict: Dict[str, C]
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]],
+    handler_dict: Dict[str, C],
 ) -> Callable[[C], C]:
     def _wrapper(func: C) -> C:
         _register_handler(file_formats, handler_dict, func)
@@ -194,11 +245,11 @@ def _registering_wrapper(
 
 
 def _register_handler(
-    file_formats: Union[str, Sequence[str]],
-    handler_dict: Dict[str, C],
+    file_formats: Union[LanguageDispatch, Sequence[LanguageDispatch]],
+    handler_dict: Dict[str, List[_DispatchRule[C]]],
     handler: C,
 ) -> None:
-    if isinstance(file_formats, str):
+    if isinstance(file_formats, LanguageDispatch):
         file_formats = [file_formats]
     else:
         if not file_formats:
@@ -206,13 +257,18 @@ def _register_handler(
         main = file_formats[0]
         for alias in file_formats[1:]:
             if alias not in _ALIAS_OF:
-                _ALIAS_OF[alias] = main
+                _ALIAS_OF[alias.language_id] = main.language_id
 
     for file_format in file_formats:
         if file_format in handler_dict:
             raise AssertionError(f"There is already a handler for {file_format}")
 
-        handler_dict[file_format] = handler
+        handler_metadata = _DispatchRule(file_format, handler)
+        handlers = handler_dict.get(file_format.language_id)
+        if handlers is None:
+            handler_dict[file_format.language_id] = [handler_metadata]
+        else:
+            handlers.append(handler_metadata)
 
 
 def ensure_lsp_features_are_loaded() -> None:
