@@ -192,8 +192,9 @@ def escape_shell(*args: str) -> str:
     return " ".join(_escape_shell_word(w) for w in args)
 
 
-def print_command(*args: str) -> None:
-    print(f"   {escape_shell(*args)}")
+def print_command(*args: str, print_at_log_level: int = logging.INFO) -> None:
+    if logging.getLogger().isEnabledFor(print_at_log_level):
+        print(f"   {escape_shell(*args)}")
 
 
 def debian_policy_normalize_symlink_target(
@@ -695,7 +696,9 @@ def package_cross_check_precheck(
 
 
 def setup_logging(
-    *, log_only_to_stderr: bool = False, reconfigure_logging: bool = False
+    *,
+    log_only_to_stderr: bool = False,
+    reconfigure_logging: bool = False,
 ) -> None:
     global _LOGGING_SET_UP, _DEFAULT_LOGGER, _STDOUT_HANDLER, _STDERR_HANDLER
     if _LOGGING_SET_UP and not reconfigure_logging:
@@ -792,7 +795,7 @@ def setup_logging(
 
     logging.setLogRecordFactory(record_factory)
 
-    logging.getLogger().setLevel(logging.INFO)
+    logging.getLogger().setLevel(logging.WARN)
     _DEFAULT_LOGGER = logging.getLogger(name)
 
     if bad_request:
