@@ -497,6 +497,44 @@ def test_dctrl_lint_sv_udeb_only(line_linter: LintWrapper) -> None:
     assert not diagnostics
 
 
+def test_dctrl_lint_udeb_menu_iten(line_linter: LintWrapper) -> None:
+    lines = textwrap.dedent(
+        """\
+    Source: foo
+    Section: devel
+    Priority: optional
+    Maintainer: Jane Developer <jane@example.com>
+    Build-Depends: debhelper-compat (= 13)
+
+    Package: foo-udeb
+    Architecture: all
+    Package-Type: udeb
+    Section: debian-installer
+    XB-Installer-Menu-Item: 12345
+    Description: Some very interesting synopsis
+     A very interesting description
+     that spans multiple lines
+     .
+     Just so be clear, this is for a test.
+
+    Package: bar-udeb
+    Architecture: all
+    Package-Type: udeb
+    Section: debian-installer
+    XB-Installer-Menu-Item: ${foo}
+    Description: Some very interesting synopsis
+     A very interesting description
+     that spans multiple lines
+     .
+     Just so be clear, this is for a test.
+    """
+    ).splitlines(keepends=True)
+
+    diagnostics = line_linter(lines)
+    print(diagnostics)
+    assert not diagnostics
+
+
 def test_dctrl_lint_multiple_vcs(line_linter: LintWrapper) -> None:
     lines = textwrap.dedent(
         f"""\
