@@ -85,6 +85,29 @@ def test_debputy_lint_unknown_keys(line_linter: LintWrapper) -> None:
     assert f"{fourth_error.range}" == "16:4-16:8"
 
 
+def test_debputy_lint_null_keys(line_linter: LintWrapper) -> None:
+    lines = textwrap.dedent(
+        """\
+    manifest-version: '0.1'
+    installations:
+    - install-docs:
+        :
+        - GETTING-STARTED-WITH-dh-debputy.md
+        - MANIFEST-FORMAT.md
+        - MIGRATING-A-DH-PLUGIN.md
+    """
+    ).splitlines(keepends=True)
+
+    diagnostics = line_linter(lines)
+    assert len(diagnostics) == 1
+    issue = diagnostics[0]
+
+    msg = "Missing key"
+    assert issue.message == msg
+    assert f"{issue.range}" == "3:4-3:5"
+    assert issue.severity == DiagnosticSeverity.Error
+
+
 @requires_levenshtein
 def test_debputy_lint_unknown_keys_spelling(line_linter: LintWrapper) -> None:
     lines = textwrap.dedent(
